@@ -148,7 +148,7 @@ static errno_t ipa_hbac_service_info_next(struct tevent_req *req,
 
     DEBUG(SSSDBG_TRACE_FUNC, ("Sending request for next search base: "
                               "[%s][%d][%s]\n", base->basedn, base->scope,
-                              base->filter));
+                              state->cur_filter));
     subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
                                    base->basedn, base->scope,
                                    state->cur_filter,
@@ -263,7 +263,7 @@ ipa_hbac_servicegroup_info_next(struct tevent_req *req,
     /* Look up service groups */
     DEBUG(SSSDBG_TRACE_FUNC, ("Sending request for next search base: "
                               "[%s][%d][%s]\n", base->basedn, base->scope,
-                              base->filter));
+                              state->cur_filter));
     subreq = sdap_get_generic_send(state, state->ev, state->opts, state->sh,
                                    base->basedn, base->scope,
                                    state->cur_filter, state->attrs, NULL, 0,
