@@ -27,5 +27,5 @@
 
 char *expand_homedir_template(TALLOC_CTX *mem_ctx, const char *template,
                               const char *username, uint32_t uid,
-                              const char *domain);
+                              const char *original, const char *domain);
 #endif
